#include "hal_data.h"
#include "coaster.h"

adc_data_size_t adc_result;
uint8_t update_result = 0;
uint8_t update_result2 = 0;
uint32_t lightsum;
uint32_t tempsum, rawtemp;
double temperature, ambient_temp;
uint8_t ADC_Data_ready;
uint8_t ADC_Timer_started;

void adc_init(void)
{
    uint8_t status;

    // Initialize ADC Driver
    status = g_adc0.p_api->open(g_adc0.p_ctrl, g_adc0.p_cfg);
    APP_ERR_TRAP(status);

    //Configure ADC Channel
    status = g_adc0.p_api->scanCfg(g_adc0.p_ctrl, g_adc0.p_channel_cfg);
    APP_ERR_TRAP(status);

    // initilize and start the ADC timer
    adc_timer_start();
}

void adc_start(void)
{
    uint8_t status;
    //Start the ADC scan
    status = g_adc0.p_api->scanStart(g_adc0.p_ctrl);
    APP_ERR_TRAP(status);
}


void g_adc0_callback(adc_callback_args_t *p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    ADC_Data_ready = 1;
}

void adc_timer_callback(timer_callback_args_t * p_args)
{
    SSP_PARAMETER_NOT_USED (p_args);
    // using this timer to start the ADC
    ADC_Timer_started = 1;
}

void adc_timer_start (void)
{
    uint8_t status;

    status = adc_timer.p_api -> open(adc_timer.p_ctrl, adc_timer.p_cfg);
    APP_ERR_TRAP(status);
    status = adc_timer.p_api ->reset(adc_timer.p_ctrl);
    APP_ERR_TRAP(status);
    status = adc_timer.p_api -> start(adc_timer.p_ctrl);
    APP_ERR_TRAP(status);
}

void read_adc_results(void)
{
    uint8_t status;
    extern adc_data_size_t light_reading;
    extern adc_data_size_t ambient_light;
    extern uint8_t first_reading;

    //Read the result
    status = g_adc0.p_api->read(g_adc0.p_ctrl, ADC_REG_CHANNEL_0, &adc_result);
    APP_ERR_TRAP(status);

    if (update_result > 4)
    {
        light_reading = (adc_data_size_t)lightsum/5; // take 5 readings and average
        update_result = 0;
        lightsum = 0;
        if (first_reading == 1)
        {
            ambient_light = light_reading;  // this happens on the very first ADC conversion only
        }
    }
    else
    {
        lightsum = lightsum + (uint32_t)adc_result;
        update_result++;
    }


    status = g_adc0.p_api->read(g_adc0.p_ctrl, ADC_REG_CHANNEL_1, &adc_result);
    APP_ERR_TRAP(status);

    if (update_result2 > 4)
    {
        rawtemp = (adc_data_size_t)tempsum/5; // take 5 readings and average

        // Linearization equation for TE thermistor assuming 12 bit ADC and 3.3V reference
        temperature = (((((float)rawtemp)/4096.0 * 3.3) - 2.5268) / (-0.0346));
        update_result2 = 0;
        tempsum = 0;
        if (first_reading == 1)
        {
            ambient_temp = temperature;  // this happens on the very first ADC conversion only
            first_reading = 0;
        }
    }
    else
    {
        tempsum = tempsum + adc_result;
        update_result2++;
    }

    //Stop the ADC scan
    status = g_adc0.p_api->scanStop(g_adc0.p_ctrl);
    APP_ERR_TRAP(status);

    ADC_Data_ready = 0;

}
